# !/usr/bin/env python
"""Init file for scannls package."""
__version__ = "0.0.1"
__PACKAGE_NAME__ = "mssw"

import mssw.cpp  # type: ignore
from mssw.cpp import Alignment, Filter  # type: ignore

import typing as t

__all__ = ["Aligner", "Alignment", "Filter"]


class Aligner:
    def __init__(
        self, match: int = 2, mismatch: int = 2, gap_open: int = 3, gap_extend: int = 1
    ):
        self._aligner = cpp.Aligner(match, mismatch, gap_open, gap_extend)

    def align(
        self,
        query,
        ref,
        align_filter: t.Optional[cpp.Filter] = None,
        alignment: t.Optional[cpp.Alignment] = None,
    ):
        if align_filter is None:
            align_filter = cpp.Filter()

        mask_len = max(len(ref) / 2, 15)

        if alignment is None:
            alignment = cpp.Alignment()
        self._aligner.Align(
            query, ref, len(ref), align_filter, alignment, int(mask_len)
        )
        return alignment



# !/usr/bin/env python
"""Build cpp extension.

@Filename:    build.py
@author:      Yangyang Li
@Time:        1/7/22 3:00 PM
"""
import platform
import typing
from pathlib import Path

from pybind11.setup_helpers import Pybind11Extension, build_ext


class MyExtension(Pybind11Extension):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)


def get_extra_options() -> typing.List[str]:
    pla_info = platform.platform().split("-")
    for item in pla_info:
        if item.startswith("arm"):
            return ["-march=armv8-a+fp+simd+crypto+crc"]
    return []


def get_files(
    path: typing.Union[Path, str], suffix: typing.List[str]
) -> typing.Iterator[str]:
    """Get bindings."""
    if isinstance(path, str):
        path = Path(path)

    for file in path.iterdir():
        if file.is_dir():
            yield from get_files(file, suffix)
        if file.suffix in suffix:
            yield file.as_posix()


def build(setup_kwargs):
    """Build cpp extension."""
    ext_modules = [
        Pybind11Extension(
            "mssw._cpp",
            language="c++",
            sources=["src/mssw/src/ssw.c", "src/mssw/src/ssw_cpp20.cpp"]
            + list(get_files("bindings", [".cpp", ".c"])),
            include_dirs=["src/mssw/src"],
            libraries=["m", "z"],
            extra_comiple_args=get_extra_options(),
        )
    ]
    setup_kwargs.update(
        {
            "ext_modules": ext_modules,
            "cmdclass": {"build_ext": build_ext},
            "zip_safe": False,
        }
    )



"""Bindings for ::StripedSmithWaterman namespace"""
from __future__ import annotations
import mssw._cpp.StripedSmithWaterman
import typing

__all__ = [
    "Aligner",
    "Alignment",
    "Filter"
]


class Aligner:
    @typing.overload
    def Align(self, query: str, filter: Filter, alignment: Alignment, maskLen: int) -> bool:
        """
        C++: StripedSmithWaterman::Aligner::Align(const char *, const struct StripedSmithWaterman::Filter &, struct StripedSmithWaterman::Alignment *, int) const --> bool

        C++: StripedSmithWaterman::Aligner::Align(const char *, const char *, int, const struct StripedSmithWaterman::Filter &, struct StripedSmithWaterman::Alignment *, int) const --> bool
        """

    @typing.overload
    def Align(self, query: str, ref: str, ref_len: int, filter: Filter, alignment: Alignment, maskLen: int) -> bool: ...

    def CleanReferenceSequence(self) -> None:
        """
        C++: StripedSmithWaterman::Aligner::CleanReferenceSequence() --> void
        """

    def Clear(self) -> None:
        """
        C++: StripedSmithWaterman::Aligner::Clear() --> void
        """

    @typing.overload
    def ReBuild(self) -> bool:
        """
        C++: StripedSmithWaterman::Aligner::ReBuild() --> bool

        C++: StripedSmithWaterman::Aligner::ReBuild(unsigned char, unsigned char, unsigned char, unsigned char) --> bool

        C++: StripedSmithWaterman::Aligner::ReBuild(const signed char *, int, const signed char *, int) --> bool
        """

    @typing.overload
    def ReBuild(self, match_score: int, mismatch_penalty: int, gap_opening_penalty: int,
                gap_extending_penalty: int) -> bool: ...

    @typing.overload
    def ReBuild(self, score_matrix: int, score_matrix_size: int, translation_matrix: int,
                translation_matrix_size: int) -> bool: ...

    def SetGapPenalty(self, opening: int, extending: int) -> None:
        """
        C++: StripedSmithWaterman::Aligner::SetGapPenalty(unsigned char, unsigned char) --> void
        """

    def SetReferenceSequence(self, seq: str, length: int) -> int:
        """
        C++: StripedSmithWaterman::Aligner::SetReferenceSequence(const char *, int) --> int
        """

    @typing.overload
    def __init__(self) -> None: ...

    @typing.overload
    def __init__(self, match_score: int, mismatch_penalty: int, gap_opening_penalty: int,
                 gap_extending_penalty: int) -> None: ...

    @typing.overload
    def __init__(self, score_matrix: int, score_matrix_size: int, translation_matrix: int,
                 translation_matrix_size: int) -> None: ...

    pass


class Alignment:
    def Clear(self) -> None:
        """
        C++: StripedSmithWaterman::Alignment::Clear() --> void
        """

    @typing.overload
    def __init__(self) -> None: ...

    @typing.overload
    def __init__(self, arg0: Alignment) -> None: ...

    @property
    def cigar(self) -> typing.List[int]:
        """
        :type: typing.List[int]
        """

    @cigar.setter
    def cigar(self, arg0: typing.List[int]) -> None:
        pass

    @property
    def cigar_string(self) -> str:
        """
        :type: str
        """

    @cigar_string.setter
    def cigar_string(self, arg0: str) -> None:
        pass

    @property
    def mismatches(self) -> int:
        """
        :type: int
        """

    @mismatches.setter
    def mismatches(self, arg0: int) -> None:
        pass

    @property
    def query_begin(self) -> int:
        """
        :type: int
        """

    @query_begin.setter
    def query_begin(self, arg0: int) -> None:
        pass

    @property
    def query_end(self) -> int:
        """
        :type: int
        """

    @query_end.setter
    def query_end(self, arg0: int) -> None:
        pass

    @property
    def ref_begin(self) -> int:
        """
        :type: int
        """

    @ref_begin.setter
    def ref_begin(self, arg0: int) -> None:
        pass

    @property
    def ref_end(self) -> int:
        """
        :type: int
        """

    @ref_end.setter
    def ref_end(self, arg0: int) -> None:
        pass

    @property
    def ref_end_next_best(self) -> int:
        """
        :type: int
        """

    @ref_end_next_best.setter
    def ref_end_next_best(self, arg0: int) -> None:
        pass

    @property
    def sw_score(self) -> int:
        """
        :type: int
        """

    @sw_score.setter
    def sw_score(self, arg0: int) -> None:
        pass

    @property
    def sw_score_next_best(self) -> int:
        """
        :type: int
        """

    @sw_score_next_best.setter
    def sw_score_next_best(self, arg0: int) -> None:
        pass

    pass


class Filter:
    @typing.overload
    def __init__(self) -> None: ...

    @typing.overload
    def __init__(self, pos: bool, cigar: bool, score: int, dis: int) -> None: ...

    @property
    def distance_filter(self) -> int:
        """
        :type: int
        """

    @distance_filter.setter
    def distance_filter(self, arg0: int) -> None:
        pass

    @property
    def report_begin_position(self) -> bool:
        """
        :type: bool
        """

    @report_begin_position.setter
    def report_begin_position(self, arg0: bool) -> None:
        pass

    @property
    def report_cigar(self) -> bool:
        """
        :type: bool
        """

    @report_cigar.setter
    def report_cigar(self, arg0: bool) -> None:
        pass

    @property
    def score_filter(self) -> int:
        """
        :type: int
        """

    @score_filter.setter
    def score_filter(self, arg0: int) -> None:
        pass

    pass



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['mssw', 'mssw.cpp']

package_data = \
{'': ['*'], 'mssw': ['src/*']}

install_requires = \
['pybind11>=2.10.0,<3.0.0', 'setuptools>=65.4.1,<66.0.0']

setup_kwargs = {
    'name': 'mssw',
    'version': '0.1.2',
    'description': 'Modern Cpp binding for complete-striped-smith-watern-library',
    'long_description': '[![pypi](https://img.shields.io/pypi/v/mssw.svg)][pypi status]\n[![status](https://img.shields.io/pypi/status/mssw.svg)][pypi status]\n[![python version](https://img.shields.io/pypi/pyversions/mssw)][pypi status]\n[![Release](https://github.com/cauliyang/Complete-Striped-Smith-Waterman-Library/actions/workflows/release.yml/badge.svg)](https://github.com/cauliyang/Complete-Striped-Smith-Waterman-Library/actions/workflows/release.yml)\n\n[pypi status]: https://pypi.org/project/mssw/0.1.1/\n\n# Modern C++ Binding for SSW Library\n\n## Changes\n\n- Add Modern C++ Binding\n- Use pybind11 Binding\n- Provide Python api\n\n## Installation\n\n```bash\n$ pip install mssw\n```\n\n## Usage\n\n### Example 1: Alignment with default filter and score matrix\n\n```python\nimport mssw\n\nreference = "CAGCCTTTCTGACCCGGAAATCAAAATAGGCACAACAAA"\nquery = "CTGAGCCGGTAAATC"\n# default match: int = 2, mismatch: int = 2, gap_open: int = 3, gap_extend: int = 1\naligner = mssw.Aligner()\naligner_filter = mssw.Filter()\nalignment = aligner.align(query, reference, aligner_filter)\n```\n\n### Example 2: Alignment with default filter and score matrix\n\n```python\nimport mssw\n\nreference = "CAGCCTTTCTGACCCGGAAATCAAAATAGGCACAACAAA"\nquery = "CTGAGCCGGTAAATC"\naligner = mssw.Aligner()\nalignment = aligner.align(query, reference)\n```\n\n### Example 3: Alignment with filter But custom gap open and gap extension\n\n```python\nimport mssw\n\nreference = "CAGCCTTTCTGACCCGGAAATCAAAATAGGCACAACAAA"\nquery = "CTGAGCCGGTAAATC"\naligner = mssw.Aligner(match=3, mismatch=1, gap_open=2, gap_extend=2)\nalignment = aligner.align(query, reference)\n```\n\n### Example 4: Alignment Result\n\n```python\nimport mssw\n\nreference = "CAGCCTTTCTGACCCGGAAATCAAAATAGGCACAACAAA"\nquery = "CTGAGCCGGTAAATC"\naligner = mssw.Aligner(match=3, mismatch=1, gap_open=2, gap_extend=2)\nalignment = aligner.align(query, reference)\n\nassert alignment.sw_score == 21\nassert alignment.sw_score_next_best == 2\nassert alignment.ref_begin == 8\nassert alignment.ref_end == 21\nassert alignment.query_begin == 0\nassert alignment.query_end == 14\nassert alignment.ref_end_next_best == 0\nassert alignment.mismatches == 2\nassert alignment.cigar_string == "4=1X4=1I5="\n```\n',
    'author': 'Yangyang Li',
    'author_email': 'yangyang.li@northwestern.edu',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}
from build import *
build(setup_kwargs)

setup(**setup_kwargs)
